/*
 * In dieser Datei werden die Expose- und Buttonevents bearbeitet.
 * Da eine Vielzahl von Moeglichkeiten vorliegen koennen, stehen alle
 * zugehoerigen Routinen in einer eigenen Datei.
 *
 */

#include	"xmik.h"

extern Display	*xdsp;
extern Window	xwin;
extern char	befehl[BEFLEN];

/*
 * Viele Mouseklicks erfordern eine genauere Untersuchung. Diese wird
 * dann in Unterprogrammen geleistet.
 *
 * Hier zunaechst die Bearbeitung eines Mouseklicks im Spielfeld.
 */

void feldbefehl(int x, int y, int b){
  int bx, by, rx, ry, zeile, spalte, bild;
  /*
   * Zuerst muss getestet werden, ob wir usn wirklich im Spielfeld
   * befinden. Innerhalb des Spielfeldes sind gewisse
   * Toleranzenmoeglich, aber die Raender sollen genau ueberprueft
   * werden.
   */

  if ((x>SKIP) && (x<(6*(SKIP+FB))) &&
      (y>SKIP) && (y<(6*(SKIP+HH)))){
    zeile=(int)(y / (SKIP+HH));
    spalte=(int)(x / (SKIP+FB));
    rx=x-(spalte*(SKIP+FB))-SKIP; ry=y-(zeile*(SKIP+HH))-SKIP;
    /* Wir befinden uns innerhalb eines Feldes und muessen nun noch
     * das Teilbild bestimmen. */
    if ((rx>0) && (ry>0)){
      bx=(int)(rx / (FB / 3));
      by=(int)(ry / (HH / 2));
      bild=3*by+bx;
      /* Letztich noch die Entscheidung, welche Taste verwendet wurde. */
      if (b==Button1) sprintf(befehl, "+%d%d%d", zeile, spalte, bild);
      else	      sprintf(befehl, "-%d%d%d", zeile, spalte, bild);}}}

/*
 * Wenn im rechten Hinweisfeld eine Taste gedrueckt wurde, ist nur zu
 * berechnen, in welchem Hinweis die Taste betaetigt wurde. Ausserdem
 * it die Tastennummer von Interesse.
 */
void rechterbefehl(int x, int y, int b){
  int rx, lx, oy, uy;

  /* Auch hier zunaechst die Untersuchung, ob wirklich innerhalb des
   * Hinweisfeldes die Taste betaetigt wurde. */
  rx=6*(SKIP+FB)+3*(SKIP+3*HB)+SKIP;
  lx=6*(SKIP+FB)+SKIP;
  oy=SKIP;
  uy=7*HH+SKIP;
  if ((x<rx) && (x>lx) && (y<uy) && (y>oy)){
    x=x-lx; y=y-oy;
    rx=(int)(x / (3*HB)); lx=(int)(y / HH);
    oy=lx+7*rx+18;
    if (b==Button1) sprintf(befehl,"h+%d",oy);
    else if (b==Button2) sprintf(befehl,"hf%d", oy);
    else	    sprintf(befehl,"h-%d",oy);}}

/*
 * Die letzte Moeglichkeit, die zu untersuchen ist, ist ein
 * Tastendruck im unteren Hinweisfeld. Sie erfolgt analog zur
 * Interpretation des rechten Hinweisfeldes.
 */
void untererbefehl(int x, int y, int b){
  int rx, lx, oy, uy;
  
  rx=SKIP+9*HB; lx=SKIP;
  oy=6*(SKIP+HH)+SKIP; uy=oy+4*HH+SKIP;
  if ((x<rx) && (x>lx) && (y<uy) && (y>oy)){
    x=x-lx; y=y-oy;
    rx=(int)(x / HB); lx=(int)(y / (2*HH));
    oy=rx+9*lx;
    if (b==Button1) sprintf(befehl,"h+%d",oy);
    else if (b==Button2) sprintf(befehl,"hf%d", oy);
    else	    sprintf(befehl,"h-%d",oy);}}

/*
 * Die Interpretation eines Buttonevents kann sehr unterschiedlich
 * ausfallen. Daher muss zu Beginn der Untersuchung dieses Events
 * festgestellt werden, in welcher Region der Event stattfand. Erst
 * danach wird unterschieden, ob die linke oder rechte Maustaste
 * betaetigt wurde. Aus all diesen Informationen wird ein
 * Befehlsstring in der Variablen `befehl' zusammengestellt. Dieser
 * Befehlsstring wird dann spaeter ausgewertet.
 */

void buttonevent(XEvent xev){
  int	x, y, b, region, rx, ry;

  /* Zuerst werden die folgenden vier Regionen unterschieden:
   * 1. Das Fenster mit dem Puzzle
   * 2. Die linke Hinweisgruppe
   * 3. Die untere Hinweisgruppe
   * 4. Die Befehlsfenster
   */

  x=xev.xbutton.x; y=xev.xbutton.y; b=xev.xbutton.button;
  region=0;

  if (x<(6*(SKIP+FB))){
    /* Wir befinden uns im linken Fensterteil */
    if (y<(6*(SKIP+HH)+SKIP)) region=1; else region=3;}
  else{
    /* Wie befinden uns im rechten Fensterteil */
    if (y<(7*HH+SKIP)) region=2; else region=4;}

  /* Nun sind die Regionen verteilt und innerhalb der Regionen kann
   * genauer untersucht werden, wo man sich befindet. */
   
  switch (region){
    /* Befehl im Spielfeld */
    case 1 : {if (!xev.xbutton.state) feldbefehl(x, y, b); break;}
    /* Befehl im rechten Hinweisfenster */
    case 2 : {rechterbefehl(x, y, b); break;}
    /* Befehl im unteren Hinweisfenster */
    case 3 : {untererbefehl(x, y, b); break;}
    /* Befehl bei einer der Schaltflaechen.
     * Hier muss nun genauer untersucht werden, welche der insgesamt
     * sieben Moeglichkeiten gewaehlt wurde. */
    case 4 : {rx=6*(SKIP+FB)+SKIP;
	      ry=7*(SKIP+HH);
	      ry=ry+HH;
	      if ((x>rx) && (x<(rx+3*HB)) && (y>ry) && (y<(ry+HH)))
		strcpy(befehl,"H");
	      ry=ry-HH;
	      if (!xev.xbutton.state){
		if ((x>rx) && (x<(rx+3*HB)) && (y>ry) && (y<(ry+HH)))
		  strcpy(befehl,"Q");
		ry=ry+2*HH;
		if ((x>rx) && (x<(rx+3*HB)) && (y>ry) && (y<(ry+HH)))
		  strcpy(befehl,"A");
		ry=ry-2*HH;
		rx=rx+3*HB;
		if ((x>rx) && (x<(rx+3*HB)) && (y>ry) && (y<(ry+HH)))
		  strcpy(befehl,"F");
		ry=ry+HH;
		if ((x>rx) && (x<(rx+3*HB)) && (y>ry) && (y<(ry+HH)))
		  strcpy(befehl,"B");
		ry=ry+HH;
		if ((x>rx) && (x<(rx+3*HB)) && (y>ry) && (y<(ry+HH)))
		  strcpy(befehl,"W");
		ry=ry-2*HH;
		rx=rx+3*HB;
		if ((x>rx) && (x<(rx+3*HB+SKIP)) && (y>ry) && (y<(ry+3*HH)))
		  strcpy(befehl,"L");
		break;}}
    default: break;}}

