/*
 * In dieser Datei finden sich die meisten Routinen, die etwas mit der
 * Initialisierung des Spiels zu tun haben. Aus organisatorischen
 * Gruenden habe ich diese Routinen von denen der Initialisierung des
 * X-Systems getrennt. Auf diese Art erschien mir die Portierung
 * einfacher moeglich.
 *
 * Der Initialisierungsprozess durchl\"auft die folgenden Stadien:
 * 1. Das Spiel wird ausgewaehlt.
 *    Dies geschieht folgendermassen. Zuerst wird untersucht, ob es
 *    einen Initialisierungsfile gibt. Ist das der Fall, dann wird
 *    dieser ausgewertet. Danach wird ueberprueft, ob ein Argument
 *    angegeben wurde. Wurde eines angegeben, dann wird dieses
 *    ausgewertet. Fuehrte keines dieser Verfahren zum Erfolg, dann
 *    wird einfach das erste Spiel gestartet.
 * 2. Der Zufallsgenerator wird initialisiert.
 * 3. Das zu loesende Puzzle wird zusammengestellt.
 * 4. Die Hinweise fuer das Puzzle werden erzeugt.
 */

#define MAXHAUFRUF 100
#include	"xmik.h"
#include	<stdlib.h>
#include	<string.h>

extern char feld[3][6][6][6];
extern char bildname[2][6][6][50];
extern int spielnummer;
extern struct tip hinweis[2][39];
extern int hinschalter;
extern int hinw1, hinw2;
extern int verschieben;
extern int drehen;
extern int gesetzte;
extern char befehl[BEFLEN], hilfestr[BEFLEN];
extern struct hist root;

extern int zahlwert(char *ziffern);
extern int zufall(int von, int bis);

extern void setze(int feldnummer, int zeile, int spalte, int wert);
extern char gesetzt(int f, int zeil, int spalte);
extern char interpretation(int feld);
extern void hilfe(void);
extern struct tip genhinw(void);
extern void fortschritt(void);


/*###################################################################
 DEBUGGING
 ##################################################################*/
void debug(void){
  int i, j, z, s;
  char zeile[80], c;
  char felder[12][50];

  for (i=0; i<12; i++){
    for (j=0; j<49; j++){
      felder[i][j]=' ';
      felder[i][49]='\0';}}

  /* Darstellung des Spielfeldes */
  for (z=0; z<6; z++){
    for (s=0; s<6; s++){
      c=gesetzt(1, z, s);
      if (c) {for (i=0; i<6; i++) felder[z*2][s*7+i+7]=c;}
      else{
        for (i=0; i<6; i++) felder[z*2][s*7+7+i]=feld[1][z][s][i];}}}

  /* Darstellung der Vorgabe */
  for (i=0; i<6; i++){
    for (j=0; j<6; j++){
      felder[i][j]=gesetzt(0, i, j);}}
  
  /* Ausgabe */
  for (i=0; i<12; i++) fprintf(stderr,"%s\n",felder[i]);
  
  for (j=0; j<79; j++) zeile[j]=' '; zeile[79]='\0';
  for (i=0; i<18; i++){
    if (hinweis[hinschalter][i].typ)
      zeile[i*3]=hinweis[hinschalter][i].typ+'0';
    else zeile[3*i]='.';}
  fprintf(stderr,"%s\n",zeile);
  for (j=0; j<79; j++) zeile[j]=' '; zeile[79]='\0';
  for (i=0; i<18; i++){
    if (hinweis[hinschalter][i].typ){
      zeile[i*3]=hinweis[hinschalter][i].www[0];
      zeile[i*3+1]=hinweis[hinschalter][i].www[1];}
    else{
      zeile[i*3]='.'; zeile[i*3+1]='.';}}
  fprintf(stderr,"%s\n",zeile);
  for (j=0; j<79; j++) zeile[j]=' '; zeile[79]='\0';
  for (i=0; i<18; i++){
    if (hinweis[hinschalter][i].typ){
      zeile[i*3]=hinweis[hinschalter][i].www[2];
      zeile[i*3+1]=hinweis[hinschalter][i].www[3];}
    else{
      zeile[i*3]='.'; zeile[i*3+1]='.';}}
  fprintf(stderr,"%s\n",zeile);

  for (j=0; j<79; j++) zeile[j]=' '; zeile[79]='\0';
  for (i=18; i<39; i++){
    if (hinweis[hinschalter][i].typ)
      zeile[(i-18)*3]=hinweis[hinschalter][i].typ+'0';
    else zeile[3*(i-18)]='.';}
  fprintf(stderr,"%s\n",zeile);
  for (j=0; j<79; j++) zeile[j]=' '; zeile[79]='\0';
  for (i=18; i<39; i++){
    if (hinweis[hinschalter][i].typ){
      zeile[(i-18)*3]=hinweis[hinschalter][i].www[0];
      zeile[(i-18)*3+1]=hinweis[hinschalter][i].www[1];}
    else{
      zeile[(i-18)*3]='.'; zeile[(i-18)*3+1]='.';}}
  fprintf(stderr,"%s\n",zeile);
  for (j=0; j<79; j++) zeile[j]=' '; zeile[79]='\0';
  for (i=18; i<39; i++){
    if (hinweis[hinschalter][i].typ){
      zeile[(i-18)*3]=hinweis[hinschalter][i].www[2];
      zeile[(i-18)*3+1]=hinweis[hinschalter][i].www[3];}
    else{
      zeile[(i-18)*3]='.'; zeile[(i-18)*3+1]='.';}}
  fprintf(stderr,"%s\n",zeile);
  for (j=0; j<79; j++) zeile[j]=' '; zeile[79]='\0';
  for (i=18; i<39; i++){
    if (hinweis[hinschalter][i].typ!=5){
      zeile[(i-18)*3]=hinweis[hinschalter][i].www[4];
      zeile[(i-18)*3+1]=hinweis[hinschalter][i].www[5];}
    else{
      zeile[(i-18)*3]='.'; zeile[(i-18)*3+1]='.';}}
  fprintf(stderr,"%s\n",zeile);
  fprintf(stderr,"\n H1: %d, H2: %d, Ges: %d\n\n",hinw1, hinw2, gesetzte);
  fprintf(stderr,"Der Befehl war: >%s<\n", befehl);}

void mydebug(char *text, int zahl){
#ifdef DEBUG
  fprintf(stderr,"%s, %d\n",text,zahl);
#endif
}

void einleitung(int argc, char *argv[]){
  FILE *fp;
  char io[100], name[100];
  int i, j, l;
  
  /* Trotz der logischen, anderslautenden Beschreibung im Kopf dieser
   * Datei wird zunaechst untersucht, ob ein Argument angegeben wurde.
   * Ist das der Fall, dann wird die Nummer des Spiels im
   * Konfigurationsfile nicht geaendert, ansonsten muss der
   * Konfigurationsfile angegelgt werden. */

  spielnummer=0;
  root.next=NULL;
  strcpy(name,BASENAME); strcat(name, CONFNAME);
  if (argc==1){
    fp=fopen(name, "r");
    if (fp){
      fgets(io, 99, fp);
      if (io[strlen(io)-1]=='\n') io[strlen(io)-1]='\0';
      spielnummer=zahlwert(io);
      fclose(fp);}
    spielnummer++;
    sprintf(io, "%d", spielnummer);
    fp=fopen(name,"w");
    fputs(io, fp);
    fclose(fp);}
  else spielnummer=zahlwert(argv[1]);
  
  /* Initialisierung des Zufallsgenerators */
  srand(spielnummer);

  /* Abschliessend werden noch den einzelnen Feldern die Bildnamen
   * zugeordnet. Diese werden alle in eine grosse Variable
   * geschrieben. */

  strcpy(name, BASENAME); strcat(name, BILDNAME);
  fp=fopen(name, "r");
  for (i=0; i<2; i++){
    for (j=0; j<6; j++){
      for (l=0; l<6; l++){
        fgets(io,99,fp);
	strcpy(bildname[i][j][l], io);
	bildname[i][j][l][strlen(bildname[i][j][l])-1]='\0';}}}}

/*
 * Hier steht der eigentliche Spielaufbau.
 * Zunaechst werden die Felder vorbesetzt, was in einem Unterprogramm
 * realisiert wird, da der Aufbau auch von anderer Stelle aus erfolgen
 * kann. Danach werden die Hinweise und die gesetzte Felder ermittelt.
 *
 * Da die einzelnen Unterprogramme i.W. von der eigentlichen Routine
 * ausa aufgerufen werden, stehen die benoetigten Hilfsprogramme auch
 * hier.
 */

  void felderbesetzen(void){
    int i, j, k, l;

    /* Normale Feldbesetzung, bei der noch nichts ausgewaehlt wurde. */
    for (i=0; i<3; i++){
      for (j=0; j<6; j++){
        for (k=0; k<6; k++){
	  for (l=0; l<6; l++) feld[i][j][k][l]=l+'0';}}}}

  void puzzleerstellen(void){
    int i, j, k, l;
    char puffer[7], h;

    /* Nach dem Zufallsprinzip wird nun eine Permutation fuer jede
     * Zeile des internen Spielfeldes ermittelt. */
    for (i=0; i<6; i++){
      strcpy(puffer,"012345");
      for (j=0; j<100; j++){
        k=zufall(0,5); l=zufall(0,5);
	h=puffer[k]; puffer[k]=puffer[l]; puffer[l]=h;}
      for (j=0; j<6; j++) setze(0, i, j, puffer[j]-'0');}}

  void hinweisenullen(void){
    int i, j;

    /* Alle Hinweise, sowie die zugehoerigen Schaltvariablen werden
     * nun in eine definierte Position gebracht. Da der Hinweistyp
     * Null nicht vorkommt, wird dieser als leerer Hinweis gesetzt. */
    for (i=0; i<39; i++){
      hinweis[0][i].typ=0; hinweis[1][i].typ=0; hinweis[2][i].typ=0;
      for (j=0; j<6; j++){
        hinweis[0][i].www[j]=' ';
	hinweis[1][i].www[j]=' ';
	hinweis[2][i].www[j]=' ';}}
    hinschalter=0;
    hinw1=hinw2=gesetzte=0;
    drehen=-1;
    verschieben=-1;}
      
void spielaufbau(int argc, char *argv[]){
  char iergebnis;
  struct tip hinw;
  int i, j, k, haufrufe;

  felderbesetzen();
  puzzleerstellen();
  hinweisenullen();
 
  iergebnis='\0'; haufrufe=0;
  do{	/* Bis Spiel fertig */

#ifdef DEBUG
fprintf(stderr,"init: Beginn der aeusseren Spielschleife.\n");
#endif

    do{	/* Bis Bedingung was bringt */

#ifdef DEBUG
fprintf(stderr,"init: Beginn der inneren Spielschleife.\n");
#endif

      if ((gesetzte == MAXSETZ) ||
	  (hinw1 == MAXUNTE) ||
	  (hinw2 == MAXRECH) ||
	  (haufrufe > MAXHAUFRUF)){

#ifdef DEBUG
fprintf(stderr,"init: Neuerstellen des Spielfeldes.\n");
#endif

        felderbesetzen();
        puzzleerstellen();
        hinweisenullen();
        iergebnis='\0';}

#ifdef DEBUG
fprintf(stderr,"init: Vor Erstellen eines Hinweises.\n");
#endif

      hinw=genhinw();
      switch (hinw.typ){
        case 0 : {gesetzte++; break;}
        case 1 : {hinw1++; break;}
        case 2 : {hinw1++; break;}
        case 3 : {hinw2++; break;}
        case 4 : {hinw2++; break;}
        case 5 : {hinw2++; break;}
        case 6 : {hinw2++; break;}
        case 7 : {hinw2++; break;}}

#ifdef DEBUG
fprintf(stderr,"Eingeordnet wird Typ: %d\n",hinw.typ);
fprintf(stderr,"An Stelle (hinw1) %d oder (hinw2) %d\n",hinw1,hinw2);
#endif
        /* Jetzt erst wird der Hinweis als akzeptabel erkannt und kann
         * in die Reihe der Hinweise eingeordnet werden. */
        if ((hinw.typ==1) || (hinw.typ==2)){
          hinweis[hinschalter][hinw1-1].typ=hinw.typ;
          for (i=0; i<6; i++)
            hinweis[hinschalter][hinw1-1].www[i]=hinw.www[i];}
        if (hinw.typ > 2){
          hinweis[hinschalter][hinw2+17].typ=hinw.typ;
          for (i=0; i<6; i++)
            hinweis[hinschalter][hinw2+17].www[i]=hinw.www[i];}
        /* Nun ist der Hinweis in die Reihe der Hinweise eingeordnet
         * und kann ausgewertet werden.
	 * Zuallererst wird nun ueberprueft, ob der Hinweis was
	 * gebracht hat. Ich das nicht der Fall, dann wird er wieder
	 * aus der Reihe der Hinweise geloescht.
	 * Diese Ueberpruefung findet aber nicht statt fuer die
	 * Hinweise der Art 3 */
	hilfe();
	if (hilfestr[0]) haufrufe=0; else haufrufe++;

#ifdef DEBUG
if (!hilfestr[0]) fprintf(stderr,"init: Keine Hilfe.\n");
else fprintf(stderr,"init: Hilfe: >%s<\n",hilfestr);
#endif

	if (!hilfestr[0]){
	  if ((hinw.typ==1) || (hinw.typ==2)){
	    hinweis[hinschalter][hinw1-1].typ=0;
	    hinw1--;}
	  if (hinw.typ > 2){
	    hinweis[hinschalter][hinw2+17].typ=0;
	    hinw2--;}}}
      while (!hilfestr[0]);
      fortschritt();
      do{
        hilfe();
        if (hilfestr[0]){
          strcpy(befehl, hilfestr);
	  iergebnis=interpretation(1);

#ifdef DEBUG
debug();
#endif
	  }}
      while ((hilfestr[0]) && (iergebnis != 'R'));}
  while (iergebnis != 'R');

  /* Nun ist das Spiel spielbar und da die Vorgaben alle auch
   * protokolliert wurden, braucht jetzt nur noch das zweite in das
   * erste Feld kopiert zu werden. */
  for (i=0; i<6; i++){
    for (j=0; j<6; j++){
      for (k=0; k<6; k++) feld[1][i][j][k]=feld[2][i][j][k];}}}

/*
 * Wurde das Puzzle nicht richtig geloest, dann muss die Nummer im
 * RC-File wieder dekrementiert werden
 */
void filedekrementieren(int argc){
  char name[100];
  FILE *fp;

  if (argc==1){
    spielnummer--;
    strcpy(name, BASENAME); strcat(name, CONFNAME);
    fp=fopen(name, "w");
    fprintf(fp,"%d",spielnummer);
    fclose(fp);}}
