/*
 * In dieser Datei wird die Interpretation der Befehlszeile
 * zusammengefasst. Zurueckgegeben wird ein Zeichen, das ggf. angibt,
 * ob die Operation zu einem Fehler gefuehrt hat. Dies wird in der
 * Historyliste vermerkt.
 *
 * Folgende Befehle werden erkannt (jeweils erstes Zeichen der
 * Befehlsvariablen):
 * Q : Verlassen des Spiels
 * H : Hilfeanforderung
 * A : Umschaltung der Hinweise
 * F : Fehler, also Zuruecknahme des letzten Befehls
 * B : Wiederholung des Spiels bis zum ersten Fehler
 * W : Wiederholung des gesamten Spiels
 * L : Logo-Befehl, freundlichen Text ausgeben
 * h : Befehl fuer die Hinweise, hier gibt es
 *     + Hinweis an andere Stelle verschieben
 *     - Hinweis in alternative Liste verschieben
 * + : Feldsymbol auswaehlen
 * - : Feldsymbol loeschen
 *
 * Abschliessend wird bei einer Veraenderung immer getestet, ob das
 * Spiel beendet wurde, ein Fehler gemacht wurde, oder ob die Aktion
 * zwar richtig war, aber noch nicht zum vollen Erfolg gefuehrt hat.
 * Aus dem Resultat ergibt sich dann der Rueckgabewert.
 */

#include	"xmik.h"
#include	<strings.h>

extern Display	*xdsp;
extern Window	xwin;
extern Cursor	xc1, xc2;
extern char	befehl[BEFLEN];
extern char	loopflag;
extern struct tip hinweis[2][39];
extern int	hinschalter;
extern int	verschieben;
extern int	drehen;
extern char	feld[3][6][6][6];
extern struct hist root;

/* Externe Unterprogramme */
extern char gesetzt(int f, int zeile, int spalte);
extern int zahlwert(char *ziffern);
extern void untenhinw(void);
extern void seitenhinw(void);
extern loescheueberfluessiges(void);
extern void dellasthist(void);
extern void filedekrementieren(int argc);

/*
 * Das nun folgende Unterprogramm testet, ob das Spiel erfolgreich
 * geloest wurde, ob ein Fehler gemacht wurde, oder ob der Spieler
 * noch auf dem richtigen Weg ist.
 */
char testefeld(int f){
  char ergebnis;
  int z, s, b;
  char fertig;

  ergebnis='\0';
  fertig=True;
  for (z=0; z<6; z++){
    for (s=0; s<6; s++){
      if (!gesetzt(1,z,s)) fertig=False;}}

  /* Wenn alle Felder gesetzt sind, dann ist nur noch die Frage, ob
   * sie denn auch alle richtig gesetzt sind. */
  if (fertig==True){
    ergebnis='R';
    for (z=0; z<6; z++){
      for (s=0; s<6; s++){
        for (b=0; b<6; b++){
	  if (feld[0][z][s][b] != feld[1][z][s][b]) ergebnis='F';}}}}

  /* Sind noch nicht alle Felder gesetzt, dann muss ueberprueft
   * werden, ob bisher ein Fehler gemacht wurde. */

  else{
    for (z=0; z<6; z++){
      for (s=0; s<6; s++){
        b=gesetzt(0, z, s);
	if (feld[f][z][s][b-'0'] != b) ergebnis='f';}}
    if (!ergebnis) ergebnis='r';}
  return(ergebnis);}

/*
 * Die folgende Routine sortiert einen Hinweis an einer neuen Stelle
 * ein. Es soll moeglichst nach hinten geschoben werden, ist das nicht
 * moeglich, wird nach vorne geschoben.
 */
void findeneuenplatz(int alt, int neu){
  int freier, i;
  struct tip sichern;

  if (alt != neu){ /* Sonst braucht man ja nix zu machen */
    if (hinweis[hinschalter][neu].typ==0){
      hinweis[hinschalter][neu]=hinweis[hinschalter][alt];
      hinweis[hinschalter][alt].typ=0;}
    else{
      sichern=hinweis[hinschalter][alt];
      hinweis[hinschalter][alt].typ=0;
      if ((alt < 18) && (neu < 18)){ /* Im unteren Hinweisfeld */
        freier=0;
        for (i=neu; i<18; i++)
	  if ((hinweis[hinschalter][i].typ==0) && (!freier))
	    freier=i;
        if (freier){
	  for (i=freier; i>neu; i--)
	    hinweis[hinschalter][i]=hinweis[hinschalter][i-1];}
        else{
	  freier=neu;
	  while (hinweis[hinschalter][freier].typ != 0) freier--;
	  for (i=freier; i<neu; i++)
	    hinweis[hinschalter][i]=hinweis[hinschalter][i+1];}}
      else if ((alt > 17) && (neu > 17)){ /* Dasselbe rechts */
        freier=0;
        for (i=neu; i<39; i++)
	  if ((hinweis[hinschalter][i].typ==0) && (!freier)) freier=i;
          if (freier){
	    for (i=freier; i>neu; i--)
	    hinweis[hinschalter][i]=hinweis[hinschalter][i-1];}
	  else{
	    freier=neu;
	    while (hinweis[hinschalter][freier].typ != 0) freier--;
	    for (i=freier; i<neu; i++)
	      hinweis[hinschalter][i]=hinweis[hinschalter][i+1];}}
	hinweis[hinschalter][neu]=sichern;}}}

/*
 * Als spezielles Feature wird hier die `Umdrehung' der Hinweise
 * realisiert.
 */
void fliphinweis(int nr){
  struct tip zwischen;
  int typ;

  typ=hinweis[hinschalter][nr].typ;
  if ((typ==1) || (typ==2)){
    zwischen=hinweis[hinschalter][nr];
    hinweis[hinschalter][nr].www[0]=zwischen.www[2];
    hinweis[hinschalter][nr].www[1]=zwischen.www[3];
    hinweis[hinschalter][nr].www[2]=zwischen.www[0];
    hinweis[hinschalter][nr].www[3]=zwischen.www[1];
    untenhinw();}
  if ((typ==3) || (typ==4)){
    zwischen=hinweis[hinschalter][nr];
    hinweis[hinschalter][nr].www[0]=zwischen.www[2];
    hinweis[hinschalter][nr].www[1]=zwischen.www[3];
    hinweis[hinschalter][nr].www[2]=zwischen.www[0];
    hinweis[hinschalter][nr].www[3]=zwischen.www[1];
    hinweis[hinschalter][nr].www[4]=zwischen.www[2];
    hinweis[hinschalter][nr].www[5]=zwischen.www[3];
    seitenhinw();}
  if ((typ==6) || (typ==7)){
    zwischen=hinweis[hinschalter][nr];
    hinweis[hinschalter][nr].www[0]=zwischen.www[4];
    hinweis[hinschalter][nr].www[1]=zwischen.www[5];
    hinweis[hinschalter][nr].www[4]=zwischen.www[0];
    hinweis[hinschalter][nr].www[5]=zwischen.www[1];
    seitenhinw();}}

/*
 * Das folgende Unterprogramm erledigt die Verschiebung der Hinweise.
 */
void hinweisbefehl(void){
  int hn, hs;
  struct tip *hilfe;

  if (hinschalter) hs=0; else hs=1;
  if (befehl[1]=='-'){
    hn=zahlwert(&befehl[2]);
    if (hn > 17) hilfe=&hinweis[hs][18]; else hilfe=&hinweis[hs][0];
    while (hilfe->typ) hilfe=hilfe++;
    *hilfe=hinweis[hinschalter][hn];
    hinweis[hinschalter][hn].typ=0;
    if (hn > 17) seitenhinw(); else untenhinw();}
  else if (befehl[1]=='+'){
    if (verschieben != -1){
      hn=zahlwert(&befehl[2]);
      findeneuenplatz(verschieben, hn);
      verschieben=-1;
      XDefineCursor(xdsp, xwin, xc1);
      if (hn > 17) seitenhinw(); else untenhinw();}
    else{
      hn=zahlwert(&befehl[2]);
      if (hinweis[hinschalter][hn].typ){
        verschieben=zahlwert(&befehl[2]);
	XDefineCursor(xdsp, xwin, xc2);}}}
  else if (befehl[1]=='f'){
    hn=zahlwert(&befehl[2]);
    if (drehen==-1){
      fliphinweis(hn);
      drehen=0;}
    else drehen=-1;}}

/*
 * Hier beginnt die eigentliche Befehlsinterpretation
 */
char interpretation(int f, int argc){
  char ergebnis, anhang[3];
  int z, s, b, i;

  ergebnis='\0';
  switch (befehl[0]){
    case 'Q' : {loopflag=False; ergebnis='\0'; filedekrementieren(argc); break;}
    case 'A' : {if (hinschalter) hinschalter=0; else hinschalter=1;
		untenhinw(); seitenhinw();
		ergebnis='\0'; break;}
    case 'L' : {printf("\nHallo, Sie spielen das Spiel\n");
		printf("          X-Mik\n");
		printf("Ein Logikspiel von F. Cremer\n\n");
		ergebnis='\0'; break;}
    case 'h' : {hinweisbefehl(); break;}
    case '+' : {z=befehl[1]-'0';
		s=befehl[2]-'0';
		b=befehl[3]-'0';
		for (i=0; i<6; i++){
		  if (i!=b) feld[f][z][s][i]=' ';
		  else      feld[f][z][s][i]=b+'0';
		  if (i != s) feld[f][z][i][b]=' ';}
		loescheueberfluessiges();
		ergebnis=testefeld(f); break;}
    case '-' : {z=befehl[1]-'0';
		s=befehl[2]-'0';
		b=befehl[3]-'0';
		feld[f][z][s][b]=' ';
		loescheueberfluessiges();
		ergebnis=testefeld(f); break;}
    default  : break;}
 
  anhang[0]='.'; anhang[1]=ergebnis; anhang[2]='\0';
  strcat(befehl,anhang);

  if ((ergebnis=='F') && (ergebnis=='f')){
    fprintf(stderr,"FEHLER\n");
    fprintf(stdout,"FEHLER\n");}
  if ((ergebnis=='F') || (ergebnis=='R')) loopflag=False;
  return(ergebnis);}
