/*
 * In dieser Datei finden sich alle Routinen zur Initialisierung des
 * X-Systems. Ausserdem wird noch das Window erzeugt.
 */

#include	"xmik.h"
#include	<X11/cursorfont.h>

/* Manche der Variablen muessen bereitgestellt werden */

extern Display		*xdsp;
extern int		xsn;
extern Cursor		xc1, xc2, xc3;
extern Colormap		xcm;
extern unsigned long	xbp, xwp;
extern Window		xwin;
extern Window		xnf, xnj, xnn;
extern GC		xgc;
extern XFontStruct	*xfi;
extern XEvent		xev;

/*
 * Initialisierung des X-Systems.
 * In dieser Routine wird die Verbundung zum X-Server hergestellt und
 * das Window erzeugt. Ausserdem werden noch ein graphischer Kontext
 * und ein Font zur Verfuegung gestellt.
 */
char xinitialisierung(int argc, char *argv[]){
  char	ergebnis;
  unsigned long	gcvm;
  XGCValues	xgcv;

  XSizeHints	xsh;
  XWMHints	xwmh;
  XClassHint	xch;
  XTextProperty	xwn, xin;
  char		*windowname="X-Mik";
  char		*iconname="X-Mik";
  XSetWindowAttributes	xswa;
  unsigned long		xavm;

  int		breite, hoehe;
  int		xpos, ypos;

  ergebnis=True;		/* Initialisierung */

  /* Oeffnen des Displays */
  xdsp=XOpenDisplay(NULL);
  if (!xdsp){
    fprintf(stderr,"Keine Verbindung zum X-Server\n");
    ergebnis=False;}
  else{

    /* Ermitteln der Screennummer und der Farbwerte, sowie
     * alloziierung der voreingestellten Colormap. */
    xsn=DefaultScreen(xdsp);
    xbp=BlackPixel(xdsp, xsn);
    xwp=WhitePixel(xdsp, xsn);
    xcm=DefaultColormap(xdsp, xsn);

    /* Test auf Breite und Hoehe des Bildschirms */
    breite=6*(SKIP+FB)+3*(SKIP+3*HB)+SKIP;
    hoehe=6*(SKIP+HH)+2*(SKIP+2*HH)+SKIP;
    if ((breite > DisplayWidth(xdsp, xsn)) ||
        (hoehe  > DisplayHeight(xdsp, xsn))){
      fprintf(stderr,"Minimale Breite und Hoehe des Bildschirms\n");
      fprintf(stderr,"muss %dx%d betragen!\n\n",breite,hoehe);
      ergebnis=False;}
    else{

      /* Nun kann das Hauptwindow erzeugt werden */
      xwin=XCreateSimpleWindow(xdsp, RootWindow(xdsp, xsn),
    			       0, 0, breite, hoehe,
			       0, xbp, xwp);
      XStringListToTextProperty(&windowname, 1, &xwn);
      XStringListToTextProperty(&iconname, 1, &xin);
      xsh.flags=USPosition | PSize;
      xsh.width=breite;
      xsh.height=hoehe;
      xsh.x=0; xsh.y=0;

      xwmh.flags=StateHint | InputHint;
      xwmh.initial_state=NormalState;
      xwmh.input=True;

      xch.res_name="X-Mik";
      xch.res_class="X-Mik";
      XSetWMProperties(xdsp, xwin, &xwn, &xin,
		       argv, argc,
		       &xsh, &xwmh, &xch);

      XSelectInput(xdsp, xwin, ExposureMask |
			       ButtonPressMask |
			       ButtonReleaseMask |
			       StructureNotifyMask);
      XMapWindow(xdsp, xwin);

      xavm=CWBackingStore;
      xswa.backing_store=Always;
      XChangeWindowAttributes(xdsp, xwin, xavm, &xswa);

      /* Nachdem nun das Window bereitsteht, kann auch der graphische
       * Kontext und der Font geladen werden. */
      gcvm=GCForeground | GCBackground | GCLineWidth;
      xgcv.foreground=xbp;
      xgcv.background=xwp;
      xgcv.line_width=3;
      xgc=XCreateGC(xdsp, xwin, gcvm, &xgcv);

      /* Nun werden die Windows fuer die Nachfrage erzeugt, aber noch
       * nicht gemappt */
      xpos=(int)((breite-400)/2);
      ypos=(int)((hoehe-100)/2);
      xnf=XCreateSimpleWindow(xdsp, xwin, xpos, ypos, 400, 100, 10, xbp, xwp);
      XChangeWindowAttributes(xdsp, xnf, xavm, &xswa);
      xnj=XCreateSimpleWindow(xdsp, xnf,  75, 35, 100, 50, 5, xbp, xwp);
      XChangeWindowAttributes(xdsp, xnj, xavm, &xswa);
      xnn=XCreateSimpleWindow(xdsp, xnf, 225, 35, 100, 50, 5, xbp, xwp);
      XChangeWindowAttributes(xdsp, xnn, xavm, &xswa);
      XSelectInput(xdsp, xnf, ButtonPressMask);
      XSelectInput(xdsp, xnj, ButtonPressMask);
      XSelectInput(xdsp, xnn, ButtonPressMask);

      xc1=XCreateFontCursor(xdsp, XC_hand2);
      xc2=XCreateFontCursor(xdsp, XC_exchange);
      xc3=XCreateFontCursor(xdsp, XC_question_arrow);
      XDefineCursor(xdsp, xwin, xc1);
      xfi=XLoadQueryFont(xdsp, "fixed");}}

  return(ergebnis);} /* Ende von xinitialisierung */

